@@ -21,6 +21,8 @@
 # SimpleDelegator's implementation serves as a nice example of the use of
 # Delegator:
 #
+#   require 'delegate'
+#
 #   class SimpleDelegator < Delegator
 #     def __getobj__
 #       @delegate_sd_obj # return object we are delegating to, required
@@ -37,6 +39,8 @@
 # Be advised, RDoc will not detect delegated methods.
 #
 class Delegator < BasicObject
+  VERSION = "0.2.0"
+
   kernel = ::Kernel.dup
   kernel.class_eval do
     alias __raise__ raise
@@ -44,7 +48,7 @@ class Delegator < BasicObject
       undef_method m
     end
     private_instance_methods.each do |m|
-      if /\Ablock_given\?\z|iterator\?\z|\A__.*__\z/ =~ m
+      if /\Ablock_given\?\z|\Aiterator\?\z|\A__.*__\z/ =~ m
         next
       end
       undef_method m
@@ -60,8 +64,8 @@ class Delegator < BasicObject
 
   ##
   # :method: raise
-  # Use __raise__ if your Delegator does not have a object to delegate the
-  # raise method call.
+  # Use #__raise__ if your Delegator does not have a object to delegate the
+  # #raise method call.
   #
 
   #
@@ -75,14 +79,14 @@ class Delegator < BasicObject
   #
   # Handles the magic of delegation through \_\_getobj\_\_.
   #
-  def method_missing(m, *args, &block)
+  ruby2_keywords def method_missing(m, *args, &block)
     r = true
     target = self.__getobj__ {r = false}
 
-    if r && target.respond_to?(m)
+    if r && target_respond_to?(target, m, false)
       target.__send__(m, *args, &block)
-    elsif ::Kernel.respond_to?(m, true)
-      ::Kernel.instance_method(m).bind(self).(*args, &block)
+    elsif ::Kernel.method_defined?(m) || ::Kernel.private_method_defined?(m)
+      ::Kernel.instance_method(m).bind_call(self, *args, &block)
     else
       super(m, *args, &block)
     end
@@ -95,14 +99,31 @@ class Delegator < BasicObject
   def respond_to_missing?(m, include_private)
     r = true
     target = self.__getobj__ {r = false}
-    r &&= target.respond_to?(m, include_private)
-    if r && include_private && !target.respond_to?(m, false)
+    r &&= target_respond_to?(target, m, include_private)
+    if r && include_private && !target_respond_to?(target, m, false)
       warn "delegator does not forward private method \##{m}", uplevel: 3
       return false
     end
     r
   end
 
+  KERNEL_RESPOND_TO = ::Kernel.instance_method(:respond_to?)
+  private_constant :KERNEL_RESPOND_TO
+
+  # Handle BasicObject instances
+  private def target_respond_to?(target, m, include_private)
+    case target
+    when Object
+      target.respond_to?(m, include_private)
+    else
+      if KERNEL_RESPOND_TO.bind_call(target, :respond_to?)
+        target.respond_to?(m, include_private)
+      else
+        KERNEL_RESPOND_TO.bind_call(target, m, include_private)
+      end
+    end
+  end
+
   #
   # Returns the methods available to this delegate object as the union
   # of this object's and \_\_getobj\_\_ methods.
@@ -201,44 +222,21 @@ class Delegator < BasicObject
     end
   end
 
-  def initialize_clone(obj) # :nodoc:
-    self.__setobj__(obj.__getobj__.clone)
+  def initialize_clone(obj, freeze: nil) # :nodoc:
+    self.__setobj__(obj.__getobj__.clone(freeze: freeze))
   end
   def initialize_dup(obj) # :nodoc:
     self.__setobj__(obj.__getobj__.dup)
   end
   private :initialize_clone, :initialize_dup
 
-  ##
-  # :method: trust
-  # Trust both the object returned by \_\_getobj\_\_ and self.
-  #
-
-  ##
-  # :method: untrust
-  # Untrust both the object returned by \_\_getobj\_\_ and self.
-  #
-
-  ##
-  # :method: taint
-  # Taint both the object returned by \_\_getobj\_\_ and self.
-  #
-
-  ##
-  # :method: untaint
-  # Untaint both the object returned by \_\_getobj\_\_ and self.
-  #
-
   ##
   # :method: freeze
   # Freeze both the object returned by \_\_getobj\_\_ and self.
   #
-
-  [:trust, :untrust, :taint, :untaint, :freeze].each do |method|
-    define_method method do
-      __getobj__.send(method)
-      super()
-    end
+  def freeze
+    __getobj__.freeze
+    super()
   end
 
   @delegator_api = self.public_instance_methods
@@ -259,6 +257,8 @@ end
 #     end
 #   end
 #
+#   require 'delegate'
+#
 #   class UserDecorator < SimpleDelegator
 #     def birth_year
 #       born_on.year
@@ -347,7 +347,7 @@ def Delegator.delegating_block(mid) # :nodoc:
   lambda do |*args, &block|
     target = self.__getobj__
     target.__send__(mid, *args, &block)
-  end
+  end.ruby2_keywords
 end
 
 #
@@ -360,6 +360,14 @@ end
 #     end
 #   end
 #
+# or:
+#
+#   MyClass = DelegateClass(ClassToDelegateTo) do    # Step 1
+#     def initialize
+#       super(obj_of_ClassToDelegateTo)              # Step 2
+#     end
+#   end
+#
 # Here's a sample of use from Tempfile which is really a File object with a
 # few special rules about storage location and when the File should be
 # deleted.  That makes for an almost textbook perfect example of how to use
@@ -383,11 +391,13 @@ end
 #     # ...
 #   end
 #
-def DelegateClass(superclass)
+def DelegateClass(superclass, &block)
   klass = Class.new(Delegator)
-  methods = superclass.instance_methods
-  methods -= ::Delegator.public_api
-  methods -= [:to_s, :inspect, :=~, :!~, :===]
+  ignores = [*::Delegator.public_api, :to_s, :inspect, :=~, :!~, :===]
+  protected_instance_methods = superclass.protected_instance_methods
+  protected_instance_methods -= ignores
+  public_instance_methods = superclass.public_instance_methods
+  public_instance_methods -= ignores
   klass.module_eval do
     def __getobj__ # :nodoc:
       unless defined?(@delegate_dc_obj)
@@ -400,15 +410,35 @@ def DelegateClass(superclass)
       __raise__ ::ArgumentError, "cannot delegate to self" if self.equal?(obj)
       @delegate_dc_obj = obj
     end
-    methods.each do |method|
+    protected_instance_methods.each do |method|
+      define_method(method, Delegator.delegating_block(method))
+      protected method
+    end
+    public_instance_methods.each do |method|
       define_method(method, Delegator.delegating_block(method))
     end
   end
   klass.define_singleton_method :public_instance_methods do |all=true|
-    super(all) - superclass.protected_instance_methods
+    super(all) | superclass.public_instance_methods
   end
   klass.define_singleton_method :protected_instance_methods do |all=true|
     super(all) | superclass.protected_instance_methods
   end
+  klass.define_singleton_method :instance_methods do |all=true|
+    super(all) | superclass.instance_methods
+  end
+  klass.define_singleton_method :public_instance_method do |name|
+    super(name)
+  rescue NameError
+    raise unless self.public_instance_methods.include?(name)
+    superclass.public_instance_method(name)
+  end
+  klass.define_singleton_method :instance_method do |name|
+    super(name)
+  rescue NameError
+    raise unless self.instance_methods.include?(name)
+    superclass.instance_method(name)
+  end
+  klass.module_eval(&block) if block
   return klass
 end
