@@ -10,7 +10,7 @@
 
 module RbConfig
   RUBY_VERSION.start_with?("3.1.") or
-    raise "ruby lib version (3.1.4) doesn't match executable version (#{RUBY_VERSION})"
+    raise "ruby lib version (3.1.6) doesn't match executable version (#{RUBY_VERSION})"
 
   # Ruby installed directory.
   TOPDIR = File.dirname(__FILE__).chomp!("/lib/ruby/3.1/sparc64-openbsd")
@@ -21,9 +21,9 @@ module RbConfig
   CONFIG["DESTDIR"] = DESTDIR
   CONFIG["MAJOR"] = "3"
   CONFIG["MINOR"] = "1"
-  CONFIG["TEENY"] = "4"
-  CONFIG["PATCHLEVEL"] = "223"
-  CONFIG["INSTALL"] = '/build/tmp/pobj//ruby-3.1.4/bin/install -c'
+  CONFIG["TEENY"] = "6"
+  CONFIG["PATCHLEVEL"] = "260"
+  CONFIG["INSTALL"] = '/usr/obj/ports/ruby-3.1.6/bin/install -c'
   CONFIG["EXEEXT"] = ""
   CONFIG["prefix"] = (TOPDIR || DESTDIR + "/usr/local")
   CONFIG["ruby_install_name"] = "$(RUBY_BASE_NAME)31"
@@ -45,7 +45,7 @@ module RbConfig
   CONFIG["RUBY_SEARCH_PATH"] = ""
   CONFIG["UNIVERSAL_INTS"] = ""
   CONFIG["UNIVERSAL_ARCHNAMES"] = ""
-  CONFIG["configure_args"] = " '--enable-shared' '--program-suffix=31' '--with-soname=ruby31' '--with-ruby-version=3.1' '--with-mantype=doc' '--with-setjmp-type=_setjmp' '--enable-pthread' '--enable-ipv6' '--without-baseruby' '--without-bundled-libffi' '--disable-option-checking' '--prefix=/usr/local' '--sysconfdir=/etc' '--mandir=/usr/local/man' '--infodir=/usr/local/info' '--localstatedir=/var' '--disable-silent-rules' '--disable-gtk-doc' 'CC=cc' 'CXX=c++' 'CFLAGS=-O2 -pipe' 'LDFLAGS=-L/usr/local/lib' 'CPPFLAGS=-DOPENSSL_NO_STATIC_ENGINE -I/usr/local/include' 'CXXFLAGS=-O2 -pipe'"
+  CONFIG["configure_args"] = " '--enable-shared' '--program-suffix=31' '--with-soname=ruby31' '--with-ruby-version=3.1' '--with-mantype=doc' '--with-setjmp-type=_setjmp' '--enable-pthread' '--enable-ipv6' '--without-baseruby' '--without-bundled-libffi' '--disable-option-checking' '--with-compress-debug-sections=no' '--prefix=/usr/local' '--sysconfdir=/etc' '--mandir=/usr/local/man' '--infodir=/usr/local/info' '--localstatedir=/var' '--disable-silent-rules' '--disable-gtk-doc' 'CC=cc' 'CXX=c++' 'CFLAGS=-O2 -pipe' 'LDFLAGS=-L/usr/local/lib' 'CPPFLAGS=-DOPENSSL_NO_STATIC_ENGINE -I/usr/local/include' 'CXXFLAGS=-O2 -pipe'"
   CONFIG["CONFIGURE"] = "configure"
   CONFIG["vendorarchdir"] = "$(vendorlibdir)/$(sitearch)"
   CONFIG["vendorlibdir"] = "$(vendordir)/$(ruby_version)"
@@ -95,10 +95,10 @@ module RbConfig
   CONFIG["rubyw_install_name"] = ""
   CONFIG["EXTDLDFLAGS"] = ""
   CONFIG["EXTLDFLAGS"] = ""
-  CONFIG["strict_warnflags"] = ""
+  CONFIG["strict_warnflags"] = "-std=gnu99"
   CONFIG["warnflags"] = "-Wall -Wextra -Wdeprecated-declarations -Wimplicit-function-declaration -Wimplicit-int -Wpointer-arith -Wwrite-strings -Wold-style-definition -Wmissing-noreturn -Wno-cast-function-type -Wno-constant-logical-operand -Wno-long-long -Wno-missing-field-initializers -Wno-overlength-strings -Wno-packed-bitfield-compat -Wno-parentheses-equality -Wno-self-assign -Wno-tautological-compare -Wno-unused-parameter -Wno-unused-value -Wunused-variable -Wundef"
   CONFIG["debugflags"] = "-ggdb3"
-  CONFIG["optflags"] = "-O3 -fno-fast-math"
+  CONFIG["optflags"] = "-fno-fast-math"
   CONFIG["NULLCMD"] = ":"
   CONFIG["ENABLE_DEBUG_ENV"] = ""
   CONFIG["DLNOBJ"] = "dln.o"
@@ -139,6 +139,7 @@ module RbConfig
   CONFIG["codesign"] = ""
   CONFIG["cleanlibs"] = ""
   CONFIG["POSTLINK"] = ":"
+  CONFIG["incflags"] = ""
   CONFIG["WERRORFLAG"] = "-Werror"
   CONFIG["CHDIR"] = "cd -P"
   CONFIG["RMALL"] = "rm -fr"
@@ -151,9 +152,9 @@ module RbConfig
   CONFIG["DOT"] = ""
   CONFIG["MAKEDIRS"] = "mkdir -p"
   CONFIG["MKDIR_P"] = "mkdir -p"
-  CONFIG["INSTALL_DATA"] = "/build/tmp/pobj//ruby-3.1.4/bin/install -c -m 644"
-  CONFIG["INSTALL_SCRIPT"] = "/build/tmp/pobj//ruby-3.1.4/bin/install -c -m 755"
-  CONFIG["INSTALL_PROGRAM"] = "/build/tmp/pobj//ruby-3.1.4/bin/install -c -s -m 755"
+  CONFIG["INSTALL_DATA"] = "/usr/obj/ports/ruby-3.1.6/bin/install -c -m 644"
+  CONFIG["INSTALL_SCRIPT"] = "/usr/obj/ports/ruby-3.1.6/bin/install -c -m 755"
+  CONFIG["INSTALL_PROGRAM"] = "/usr/obj/ports/ruby-3.1.6/bin/install -c -s -m 755"
   CONFIG["SET_MAKE"] = ""
   CONFIG["LN_S"] = "ln -s"
   CONFIG["DLLWRAP"] = ""
@@ -163,7 +164,7 @@ module RbConfig
   CONFIG["try_header"] = ""
   CONFIG["CC_VERSION_MESSAGE"] = "cc (GCC) 4.2.1 20070719 \nCopyright (C) 2007 Free Software Foundation, Inc.\nThis is free software; see the source for copying conditions.  There is NO\nwarranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE."
   CONFIG["CC_VERSION"] = "$(CC) --version"
-  CONFIG["MJIT_CC"] = "/usr/bin/cc -std=gnu99"
+  CONFIG["MJIT_CC"] = "/usr/bin/cc"
   CONFIG["CSRCFLAG"] = ""
   CONFIG["COUTFLAG"] = "-o "
   CONFIG["OUTFLAG"] = "-o "
@@ -185,25 +186,25 @@ module RbConfig
   CONFIG["CXX"] = "c++"
   CONFIG["AS"] = "as"
   CONFIG["AR"] = "ar"
-  CONFIG["CC"] = "cc -std=gnu99"
-  CONFIG["target_os"] = "openbsd7.3"
+  CONFIG["CC"] = "cc"
+  CONFIG["target_os"] = "openbsd7.6"
   CONFIG["target_vendor"] = "unknown"
   CONFIG["target_cpu"] = "sparc64"
-  CONFIG["target"] = "sparc64-unknown-openbsd7.3"
-  CONFIG["host_os"] = "openbsd7.3"
+  CONFIG["target"] = "sparc64-unknown-openbsd7.6"
+  CONFIG["host_os"] = "openbsd7.6"
   CONFIG["host_vendor"] = "unknown"
   CONFIG["host_cpu"] = "sparc64"
-  CONFIG["host"] = "sparc64-unknown-openbsd7.3"
-  CONFIG["build_os"] = "openbsd7.3"
+  CONFIG["host"] = "sparc64-unknown-openbsd7.6"
+  CONFIG["build_os"] = "openbsd7.6"
   CONFIG["build_vendor"] = "unknown"
   CONFIG["build_cpu"] = "sparc64"
-  CONFIG["build"] = "sparc64-unknown-openbsd7.3"
+  CONFIG["build"] = "sparc64-unknown-openbsd7.6"
   CONFIG["RUBY_VERSION_NAME"] = "$(RUBY_BASE_NAME)-$(ruby_version)"
   CONFIG["RUBYW_BASE_NAME"] = "rubyw"
   CONFIG["RUBY_BASE_NAME"] = "ruby"
   CONFIG["RUBY_PROGRAM_VERSION"] = "$(MAJOR).$(MINOR).$(TEENY)"
   CONFIG["RUBY_API_VERSION"] = "$(MAJOR).$(MINOR)"
-  CONFIG["HAVE_GIT"] = "yes"
+  CONFIG["HAVE_GIT"] = "no"
   CONFIG["GIT"] = "git"
   CONFIG["cxxflags"] = ""
   CONFIG["cppflags"] = ""
