@@ -1,3 +1,5 @@
+# frozen_string_literal: true
+
 module RBS
   class DefinitionBuilder
     attr_reader :env
@@ -10,11 +12,11 @@ module RBS
     attr_reader :singleton0_cache
     attr_reader :interface_cache
 
-    def initialize(env:)
+    def initialize(env:, ancestor_builder: nil, method_builder: nil)
       @env = env
       @type_name_resolver = TypeNameResolver.from_env(env)
-      @ancestor_builder = AncestorBuilder.new(env: env)
-      @method_builder = MethodBuilder.new(env: env)
+      @ancestor_builder = ancestor_builder || AncestorBuilder.new(env: env)
+      @method_builder = method_builder || MethodBuilder.new(env: env)
 
       @instance_cache = {}
       @singleton_cache = {}
@@ -80,6 +82,7 @@ module RBS
                      when AST::Members::Alias
                        unless definition.methods.key?(original.old_name)
                          raise UnknownMethodAliasError.new(
+                           type_name: type_name,
                            original_name: original.old_name,
                            aliased_name: original.new_name,
                            location: original.location
@@ -139,9 +142,8 @@ module RBS
         case entry
         when Environment::ClassEntry, Environment::ModuleEntry
           ancestors = ancestor_builder.instance_ancestors(type_name)
-          self_type = Types::ClassInstance.new(name: type_name,
-                                               args: Types::Variable.build(entry.type_params.each.map(&:name)),
-                                               location: nil)
+          args = entry.type_params.map {|param| Types::Variable.new(name: param.name, location: param.location) }
+          self_type = Types::ClassInstance.new(name: type_name, args: args, location: nil)
 
           Definition.new(type_name: type_name, entry: entry, self_type: self_type, ancestors: ancestors).tap do |definition|
             one_ancestors = ancestor_builder.one_instance_ancestors(type_name)
@@ -173,11 +175,15 @@ module RBS
                          end
 
                   # Successor interface method overwrites.
-                  merge_definition(src: defn,
-                                   dest: definition,
-                                   subst: Substitution.build(defn.type_params, ans.args),
-                                   keep_super: true)
+                  merge_definition(
+                    src: defn,
+                    dest: definition,
+                    subst: Substitution.build(defn.type_params, ans.args),
+                    keep_super: true
+                  )
                 end
+              else
+                methods_with_self = build_instance(type_name, no_self_types: false).methods
               end
             end
 
@@ -211,12 +217,20 @@ module RBS
               end
             end
 
-            define_methods(definition,
-                           interface_methods: interface_methods,
-                           methods: methods,
-                           super_interface_method: entry.is_a?(Environment::ModuleEntry))
+            if entry.is_a?(Environment::ModuleEntry)
+              define_methods_module_instance(
+                definition,
+                methods: methods,
+                interface_methods: interface_methods,
+                module_self_methods: methods_with_self
+              )
+            else
+              define_methods_instance(definition, methods: methods, interface_methods: interface_methods)
+            end
 
             entry.decls.each do |d|
+              subst = Substitution.build(d.decl.type_params.each.map(&:name), args)
+
               d.decl.members.each do |member|
                 case member
                 when AST::Members::AttrReader, AST::Members::AttrAccessor, AST::Members::AttrWriter
@@ -229,12 +243,18 @@ module RBS
                                 end
 
                     if ivar_name
-                      insert_variable(type_name, definition.instance_variables, name: ivar_name, type: member.type)
+                      insert_variable(type_name,
+                                      definition.instance_variables,
+                                      name: ivar_name,
+                                      type: member.type.sub(subst))
                     end
                   end
 
                 when AST::Members::InstanceVariable
-                  insert_variable(type_name, definition.instance_variables, name: member.name, type: member.type)
+                  insert_variable(type_name,
+                                  definition.instance_variables,
+                                  name: member.name,
+                                  type: member.type.sub(subst))
 
                 when AST::Members::ClassVariable
                   insert_variable(type_name, definition.class_variables, name: member.name, type: member.type)
@@ -283,6 +303,10 @@ module RBS
             end
 
             one_ancestors.each_extended_module do |mod|
+              mod.args.each do |arg|
+                validate_type_presence(arg)
+              end
+
               mod_defn = build_instance(mod.name, no_self_types: true)
               merge_definition(src: mod_defn,
                                dest: definition,
@@ -291,6 +315,10 @@ module RBS
 
             interface_methods = {}
             one_ancestors.each_extended_interface do |mod|
+              mod.args.each do |arg|
+                validate_type_presence(arg)
+              end
+
               mod_defn = build_interface(mod.name)
               subst = Substitution.build(mod_defn.type_params, mod.args)
 
@@ -312,7 +340,7 @@ module RBS
             end
 
             methods = method_builder.build_singleton(type_name)
-            define_methods(definition, interface_methods: interface_methods, methods: methods, super_interface_method: false)
+            define_methods_singleton(definition, methods: methods, interface_methods: interface_methods)
 
             entry.decls.each do |d|
               d.decl.members.each do |member|
@@ -353,62 +381,75 @@ module RBS
         when Environment::ClassEntry, Environment::ModuleEntry
           ancestors = ancestor_builder.singleton_ancestors(type_name)
           self_type = Types::ClassSingleton.new(name: type_name, location: nil)
-          instance_type = Types::ClassInstance.new(
-            name: type_name,
-            args: entry.type_params.each.map { Types::Bases::Any.new(location: nil) },
-            location: nil
-          )
 
           Definition.new(type_name: type_name, entry: entry, self_type: self_type, ancestors: ancestors).tap do |definition|
             def0 = build_singleton0(type_name)
-            subst = Substitution.build([], [], instance_type: instance_type)
+            subst = Substitution.new
 
             merge_definition(src: def0, dest: definition, subst: subst, keep_super: true)
 
             if entry.is_a?(Environment::ClassEntry)
               new_method = definition.methods[:new]
               if new_method.defs.all? {|d| d.defined_in == BuiltinNames::Class.name }
+                alias_methods = definition.methods.each.with_object([]) do |entry, array|
+                  # @type var method: Definition::Method?
+                  name, method = entry
+                  while method
+                    if method.alias_of == new_method
+                      array << name
+                      break
+                    end
+                    method = method.alias_of
+                  end
+                end
+
                 # The method is _untyped new_.
 
                 instance = build_instance(type_name)
                 initialize = instance.methods[:initialize]
 
                 if initialize
-                  class_params = entry.type_params.each.map(&:name)
+                  class_params = entry.type_params
 
                   # Inject a virtual _typed new_.
                   initialize_defs = initialize.defs
-                  definition.methods[:new] = Definition::Method.new(
+                  typed_new = Definition::Method.new(
                     super_method: new_method,
                     defs: initialize_defs.map do |initialize_def|
                       method_type = initialize_def.type
 
-                      class_type_param_vars = Set.new(class_params)
-                      method_type_param_vars = Set.new(method_type.type_params)
+                      class_type_param_vars = Set.new(class_params.map(&:name))
+                      method_type_param_vars = Set.new(method_type.type_params.map(&:name))
 
                       if class_type_param_vars.intersect?(method_type_param_vars)
-                        renamed_method_params = method_type.type_params.map do |name|
-                          if class_type_param_vars.include?(name)
-                            Types::Variable.fresh(name).name
+                        new_method_param_names = method_type.type_params.map do |method_param|
+                          if class_type_param_vars.include?(method_param.name)
+                            Types::Variable.fresh(method_param.name).name
                           else
-                            name
+                            method_param.name
                           end
                         end
-                        method_params = class_params + renamed_method_params
 
-                        sub = Substitution.build(method_type.type_params, Types::Variable.build(renamed_method_params))
+                        sub = Substitution.build(
+                          method_type.type_params.map(&:name),
+                          Types::Variable.build(new_method_param_names)
+                        )
+
+                        method_params = class_params + AST::TypeParam.rename(method_type.type_params, new_names: new_method_param_names)
+                        method_type = method_type
+                          .update(type_params: [])
+                          .sub(sub)
+                          .update(type_params: method_params)
                       else
-                        method_params = class_params + method_type.type_params
-                        sub = Substitution.build([], [])
+                        method_type = method_type
+                          .update(type_params: class_params + method_type.type_params)
                       end
 
-                      method_type = method_type.map_type {|ty| ty.sub(sub) }
                       method_type = method_type.update(
-                        type_params: method_params,
                         type: method_type.type.with_return_type(
                           Types::ClassInstance.new(
                             name: type_name,
-                            args: Types::Variable.build(entry.type_params.each.map(&:name)),
+                            args: entry.type_params.map {|param| Types::Variable.new(name: param.name, location: param.location) },
                             location: nil
                           )
                         )
@@ -425,6 +466,15 @@ module RBS
                     annotations: [],
                     alias_of: nil
                   )
+
+                  definition.methods[:new] = typed_new
+
+                  alias_methods.each do |alias_name|
+                    definition.methods[alias_name] = definition.methods[alias_name].update(
+                      alias_of: typed_new,
+                      defs: typed_new.defs
+                    )
+                  end
                 end
               end
             end
@@ -435,7 +485,7 @@ module RBS
 
     def validate_params_with(type_params, result:)
       type_params.each do |param|
-        unless param.skip_validation
+        unless param.unchecked?
           unless result.compatible?(param.name, with_annotation: param.variance)
             yield param
           end
@@ -522,8 +572,12 @@ module RBS
 
         if method_types
           method_types.each do |method_type|
+            merged_params = type_params
+              .reject {|param| method_type.type_param_names.include?(param.name) }
+              .concat(method_type.type_params)
+
             result = calculator.in_method_type(method_type: method_type, variables: param_names)
-            validate_params_with(type_params, result: result) do |param|
+            validate_params_with(merged_params, result: result) do |param|
               raise InvalidVarianceAnnotationError.new(
                 type_name: definition.type_name,
                 param: param,
@@ -543,17 +597,45 @@ module RBS
       )
     end
 
-    def define_methods(definition, interface_methods:, methods:, super_interface_method:)
+    def define_methods_instance(definition, methods:, interface_methods:)
+      define_methods(
+        definition,
+        methods: methods,
+        interface_methods: interface_methods,
+        methods_with_self: nil,
+        super_interface_method: false
+      )
+    end
+
+    def define_methods_module_instance(definition, methods:, interface_methods:, module_self_methods:)
+      define_methods(definition, methods: methods, interface_methods: interface_methods, methods_with_self: module_self_methods, super_interface_method: true)
+    end
+
+    def define_methods_singleton(definition, methods:, interface_methods:)
+      define_methods(
+        definition,
+        methods: methods,
+        interface_methods: interface_methods,
+        methods_with_self: nil,
+        super_interface_method: false
+      )
+    end
+
+    def define_methods(definition, methods:, interface_methods:, methods_with_self:, super_interface_method:)
       methods.each do |method_def|
         method_name = method_def.name
         original = method_def.original
 
         if original.is_a?(AST::Members::Alias)
           existing_method = interface_methods[method_name] || definition.methods[method_name]
-          original_method = interface_methods[original.old_name] || definition.methods[original.old_name]
+          original_method =
+            interface_methods[original.old_name] ||
+            methods_with_self&.[](original.old_name) ||
+            definition.methods[original.old_name]
 
           unless original_method
             raise UnknownMethodAliasError.new(
+              type_name: definition.type_name,
               original_name: original.old_name,
               aliased_name: original.new_name,
               location: original.location
@@ -565,7 +647,7 @@ module RBS
             defs: original_method.defs.map do |defn|
               defn.update(defined_in: definition.type_name, implemented_in: definition.type_name)
             end,
-            accessibility: method_def.accessibility,
+            accessibility: original_method.accessibility,
             alias_of: original_method
           )
         else
@@ -723,11 +805,27 @@ module RBS
     end
 
     def merge_method(type_name, methods, name, method, sub, implemented_in: :keep, keep_super: false)
-      defs = method.defs.yield_self do |defs|
+      if sub.empty? && implemented_in == :keep && keep_super
+        methods[name] = method
+      else
         if sub.empty? && implemented_in == :keep
-          defs
+          defs = method.defs
         else
-          defs.map do |defn|
+          defs = method.defs.map do |defn|
+            defn.update(
+              type: sub.empty? ? defn.type : defn.type.sub(sub),
+              implemented_in: case implemented_in
+                              when :keep
+                                defn.implemented_in
+                              when nil
+                                nil
+                              else
+                                implemented_in
+                              end
+            )
+          end
+
+          defs = method.defs.map do |defn|
             defn.update(
               type: sub.empty? ? defn.type : defn.type.sub(sub),
               implemented_in: case implemented_in
@@ -741,45 +839,98 @@ module RBS
             )
           end
         end
+
+        super_method = methods[name]
+
+        methods[name] = Definition::Method.new(
+          super_method: keep_super ? method.super_method : super_method,
+          accessibility: method.accessibility,
+          defs: defs,
+          alias_of: method.alias_of
+        )
       end
-
-      super_method = methods[name]
-
-      methods[name] = Definition::Method.new(
-        super_method: keep_super ? method.super_method : super_method,
-        accessibility: method.accessibility,
-        defs: defs,
-        alias_of: method.alias_of
-      )
     end
 
-    def try_cache(type_name, cache:, key: type_name)
-      # @type var cc: Hash[untyped, Definition | false | nil]
+    def try_cache(type_name, cache:, key: nil)
+      # @type var cc: Hash[untyped, Definition | nil]
+      # @type var key: untyped
+      key ||= type_name
       cc = _ = cache
-      cached = cc[key]
 
-      case cached
-      when Definition
-        cached
-      when false
-        raise
-      when nil
-        cc[key] = false
-        begin
-          cc[key] = yield
-        rescue => ex
-          cc.delete(key)
-          raise ex
-        end
-      else
-        raise
-      end
+      cc[key] ||= yield
     end
 
     def expand_alias(type_name)
-      entry = env.alias_decls[type_name] or raise "Unknown name for expand_alias: #{type_name}"
+      expand_alias2(type_name, [])
+    end
+
+    def expand_alias1(type_name)
+      entry = env.alias_decls[type_name] or raise "Unknown alias name: #{type_name}"
+      as = entry.decl.type_params.each.map { Types::Bases::Any.new(location: nil) }
+      expand_alias2(type_name, as)
+    end
+
+    def expand_alias2(type_name, args)
+      entry = env.alias_decls[type_name] or raise "Unknown alias name: #{type_name}"
+
       ensure_namespace!(type_name.namespace, location: entry.decl.location)
-      entry.decl.type
+      params = entry.decl.type_params.each.map(&:name)
+
+      unless params.size == args.size
+        as = "[#{args.join(", ")}]" unless args.empty?
+        ps = "[#{params.join(", ")}]" unless params.empty?
+
+        raise "Invalid type application: type = #{type_name}#{as}, decl = #{type_name}#{ps}"
+      end
+
+      type = entry.decl.type
+
+      unless params.empty?
+        subst = Substitution.build(params, args)
+        type = type.sub(subst)
+      end
+
+      type
+    end
+
+    def update(env:, except:, ancestor_builder:)
+      method_builder = self.method_builder.update(env: env, except: except)
+
+      DefinitionBuilder.new(env: env, ancestor_builder: ancestor_builder, method_builder: method_builder).tap do |builder|
+        builder.instance_cache.merge!(instance_cache)
+        builder.singleton_cache.merge!(singleton_cache)
+        builder.singleton0_cache.merge!(singleton0_cache)
+        builder.interface_cache.merge!(interface_cache)
+
+        except.each do |name|
+          builder.instance_cache.delete([name, true])
+          builder.instance_cache.delete([name, false])
+          builder.singleton_cache.delete(name)
+          builder.singleton0_cache.delete(name)
+          builder.interface_cache.delete(name)
+        end
+      end
+    end
+
+    def validate_type_presence(type)
+      case type
+      when Types::ClassInstance, Types::ClassSingleton, Types::Interface, Types::Alias
+        validate_type_name(type.name, type.location)
+      end
+
+      type.each_type do |type|
+        validate_type_presence(type)
+      end
+    end
+
+    def validate_type_name(name, location)
+      name = name.absolute!
+
+      return if name.class? && env.class_decls.key?(name)
+      return if name.interface? && env.interface_decls.key?(name)
+      return if name.alias? && env.alias_decls.key?(name)
+
+      raise NoTypeFoundError.new(type_name: name, location: location)
     end
   end
 end
